/*
 * Streaming PWC coder.
 * Paul Ausbeck 4/06/98
 *
 */

#define _INC_PWC

#ifndef _INC_IFILELIB
#include "ifilelib.h"
#endif

#ifndef _INC_PWCFILE
#include "pwcfile.h"
#endif

class pwc_recommended_numbers
{
public:
  enum { maxlps = 7 } numbers;
};

/* Encoder/Decoder external interface. */

class binary_coder;
class predictive_coder;
class pwc_base;

class pwc_coder 
{
  binary_coder *codes;
  predictive_coder *pcoder;
  pwc_base *base;

public:
  /* Construct encoder. */
  pwc_coder(int rows, int columns, int depth, int colors,
            void *handle, int (*row_function)(void *handle, 
                                              unsigned char *buffer, int columns), 
            ifile_io_wrapper *wrapper, pwc_options &options);

  /* Construct decoder. */
  pwc_coder(int rows, int columns, int depth, int colors,
            void *handle, int (*row_function)(void *handle, 
                                              unsigned char *buffer, int columns),
            ifile_io_wrapper *wrapper);

  /*
     I/O is accomplished via the callback routines passed to the constructor.

     The io_wrapper can be any anonymous I/O mechanism. See the ifilelib.h header 
     file for details. The io_wrapper is used to manage header information in
     a PWC data block.

     The row function is called to get or put row data. Multiple pixels are packed
     into each data byte when the palette depth is four or one. The handle is
     passed as an argument to the row function. The buffer is padded out to 
     modulo four bytes. The callback function should not have to worry about 
     this padding. It is available to enable efficient interface with file formats
     such as the Windows-OS/2 bitmap (BMP) file.
  */

  ~pwc_coder(void);

  static int header_size(void) { return pwc_header::header_size(); }
  /*
     Do not construct a pwc_coder unless at least header_size() bytes are immediately 
     available to the file wrapper. This guard need only be implemented in push mode
     applications such as internet browsers.
  */

  int code_header(ifile_io_wrapper *wrapper); /* Returns non-zero if data pending. */

  int code_row(void); /* Code a single row. */
                      /* When decoding, make sure that enough data is available to 
                         ensure complete processing of the scanline before calling
                         this function. Twice the unencoded scanline size should be
                         sufficient. See code with callback below. */

  int code(void); /* Code entire image. */

  /* Code while bytes available > columns*depth/4, essentially twice the number of
     bytes in an unencoded scanline. If callback is null, code until complete. */
  int code(void *handle, int (*buffered_bytes_available)(void *handle));
  void flush(void); // Flush the arithmetic coder.
                    // Alternately, delete the pwc_coder object.
                    // If flush is called before deletion, the destructor won't flush.
};
